@@ -5,9 +5,9 @@ use petgraph::{graph::NodeIndex, stable_graph::StableGraph};
 use ra_ap_hir::{self, Crate};
 use ra_ap_ide::{AnalysisHost, RootDatabase};
 use ra_ap_paths::AbsPathBuf;
-use ra_ap_proc_macro_api::ProcMacroClient;
+use ra_ap_proc_macro_api::ProcMacroServer;
 use ra_ap_project_model::{
-    CargoConfig, PackageData, ProjectManifest, ProjectWorkspace, TargetData,
+    CargoConfig, PackageData, ProjectManifest, ProjectWorkspace, TargetData, UnsetTestCrates,
 };
 use ra_ap_rust_analyzer::cli::load_cargo::{load_workspace, LoadCargoConfig};
 use ra_ap_vfs::Vfs;
@@ -47,6 +47,13 @@ pub enum Command {
 }
 
 impl Command {
+    pub(crate) fn sanitize(&mut self) {
+        if self.graph_options().with_tests && !self.project_options().cfg_test {
+            debug!("Enabling `--cfg-test`, which is implied by `--with-tests`");
+            self.project_options_mut().cfg_test = true;
+        }
+    }
+
     pub fn run(&self) -> Result<(), anyhow::Error> {
         let general_options = self.general_options();
         let project_options = self.project_options();
@@ -87,7 +94,7 @@ impl Command {
 
         trace!("Generating ...");
 
-        match &self {
+        match self {
             #[allow(unused_variables)]
             Self::Tree(options) => {
                 let command = tree::Command::new(options.clone());
@@ -122,7 +129,10 @@ impl Command {
         let rustc_source = None;
 
         // crates to disable `#[cfg(test)]` on
-        let unset_test_crates = vec![];
+        let unset_test_crates = match project_options.cfg_test {
+            true => UnsetTestCrates::None,
+            false => UnsetTestCrates::All,
+        };
 
         // Setup RUSTC_WRAPPER to point to `rust-analyzer` binary itself.
         // (We use that to compile only proc macros and build scripts
@@ -144,7 +154,7 @@ impl Command {
     fn load_config(&self) -> LoadCargoConfig {
         LoadCargoConfig {
             load_out_dirs_from_check: true,
-            with_proc_macro: true,
+            with_proc_macro: false,
             prefill_caches: false,
         }
     }
@@ -155,9 +165,6 @@ impl Command {
         cargo_config: &CargoConfig,
         progress: &dyn Fn(String),
     ) -> anyhow::Result<ProjectWorkspace> {
-        // let project_path = project_options.manifest_path.as_path().canonicalize()?;
-        // let cargo_config = Self::cargo_config(project_options);
-
         let root = AbsPathBuf::assert(std::env::current_dir()?.join(project_path));
         let root = ProjectManifest::discover_single(&root)?;
 
@@ -190,12 +197,17 @@ impl Command {
 
     fn analyze_project_workspace(
         &self,
-        project_workspace: ProjectWorkspace,
+        mut workspace: ProjectWorkspace,
         cargo_config: &CargoConfig,
         load_config: &LoadCargoConfig,
         progress: &dyn Fn(String),
-    ) -> anyhow::Result<(AnalysisHost, Vfs, Option<ProcMacroClient>)> {
-        load_workspace(project_workspace, cargo_config, load_config, &progress)
+    ) -> anyhow::Result<(AnalysisHost, Vfs, Option<ProcMacroServer>)> {
+        if load_config.load_out_dirs_from_check {
+            let build_scripts = workspace.run_build_scripts(cargo_config, progress)?;
+            workspace.set_build_scripts(build_scripts)
+        }
+
+        load_workspace(workspace, load_config)
     }
 
     fn find_crate(
@@ -255,7 +267,7 @@ impl Command {
     }
 
     fn with_sysroot(&self) -> bool {
-        match &self {
+        match self {
             Self::Tree(_) => false,
             Self::Graph(options) => {
                 // We only need to include sysroot if we include extern uses
@@ -266,28 +278,52 @@ impl Command {
     }
 
     fn general_options(&self) -> &GeneralOptions {
-        match &self {
+        match self {
             Self::Tree(options) => &options.general,
             Self::Graph(options) => &options.general,
         }
     }
 
+    #[allow(dead_code)]
+    fn general_options_mut(&mut self) -> &mut GeneralOptions {
+        match self {
+            Self::Tree(options) => &mut options.general,
+            Self::Graph(options) => &mut options.general,
+        }
+    }
+
     fn project_options(&self) -> &ProjectOptions {
-        match &self {
+        match self {
             Self::Tree(options) => &options.project,
             Self::Graph(options) => &options.project,
         }
     }
 
+    #[allow(dead_code)]
+    fn project_options_mut(&mut self) -> &mut ProjectOptions {
+        match self {
+            Self::Tree(options) => &mut options.project,
+            Self::Graph(options) => &mut options.project,
+        }
+    }
+
     fn graph_options(&self) -> &GraphOptions {
-        match &self {
+        match self {
             Self::Tree(options) => &options.graph,
             Self::Graph(options) => &options.graph,
         }
     }
 
+    #[allow(dead_code)]
+    fn graph_options_mut(&mut self) -> &mut GraphOptions {
+        match self {
+            Self::Tree(options) => &mut options.graph,
+            Self::Graph(options) => &mut options.graph,
+        }
+    }
+
     fn builder_options(&self) -> GraphBuilderOptions {
-        match &self {
+        match self {
             Self::Tree(options) => GraphBuilderOptions {
                 focus_on: options.graph.focus_on.clone(),
                 max_depth: options.graph.max_depth,
